function [ OutCorr2Sqr, OutRMSE, NewDes, NewRes ] = EnsembleAveragingSVM( TestRes )

% This function sorts the data to have the same encodings in the same rows and gives back the output of each SVM in the ensemble average.
% This is necessary because the ensemble average is built with SVM trained with different pattern grouping in the n-fold cross validation
% This is a validation result. To get the final SVM models please use "GetFinalSVM.m"

% Check that NewDes has the values of the first column repeated for all other columns which means that the sorting is correct

% TestRes comes from "FinalTestResultsSVM.m"

% NewRes            Contains the results for each SVM. Contains in rows each different protein encoding and in columns the number of SVM composing the ensemble
% NewDes            Contains the desired values/protein expression for each SVM. Contains in rows each different protein encoding and in columns the number of SVM composing the ensemble
% OutRMSE           Root mean squared error of the ensemble
% OutCorr2Sqr       Squared corelation coefficient of the ensemble


startDir = pwd;

sizeTestRes = size(TestRes,2);

for i=1:sizeTestRes; Res(:,i)=TestRes(i).SumAllRepTest; end

for i=1:sizeTestRes; Des(:,i)=TestRes(i).DesiredTest; end

[ OutSeq ] = FindSeq;    

for i=1:sizeTestRes; Undo(:,i) = UndoSeq( OutSeq(i).Seq )'; end

for i=1:sizeTestRes; NewDes(:,i) = Des( Undo(:,i),i ); end

for i=1:sizeTestRes; NewRes(:,i) = Res( Undo(:,i),i ); end

OutCorr2Sqr = corr2( sum(NewRes,2)/sizeTestRes , NewDes(:,1)).^2;

OutRMSE = sqrt(mean(( sum(NewRes,2)/sizeTestRes - NewDes(:,1)).^2));

cd(startDir)


end

