function [ ResInd, ResEns, NewData ] = FinalEnsembleAveragingResults(mode)

% This function calculates the results of ensemble averaging for both Welch and Kudla

% mode  1 - Welch    2 - Kudla

% ResInd contains the results for the individual SVM used for ensemble averaging
% ResEns contains the result from ensemble averaging
% NewData contains input features and desired values that correspond to protein expression levels

% By changing the files loaded it is possible to test the ensemble averaging model with new protein encodings.

% Please be aware that the models were created for the proteins from Welch et al. or Kudla/Supek and Smuc works and do not work for other proteins 

% Use this software at your own risk. No warranty is provided.

clc;

%% set Welch
if mode==1

load('Data_Codon_WithNorm_MFE.mat')

SelPat = Data.Used & ~isnan(Data.Des) ;

NewData.InTemp = Data.In( SelPat ,:);

NewData.CodonCount =  codoncountArray(  Data.Sequence( SelPat ,:)  );
[ ~, NewData.Codon ] = nt2codons(  Data.Sequence( SelPat ,:)  );           
[ NewData.CodonBias , ~ ] = CodonBiasArray( Data.Sequence( SelPat ) );  

NewData.In = [NewData.InTemp(:,9) NewData.CodonBias(:,1:end-3) NewData.Codon NewData.CodonCount];

load('Welch_FinalModels.mat')
 
%% Kudla
elseif mode==2

load('Kudla_Dataset_WithEnergy.mat')

[ NewData.CodonBias , ~ ] = CodonBiasArray( Data.Sequence );  
[ ~ , NewData.Codon ] = nt2codons( Data.Sequence );  
NewData.CodonCount =  codoncountArray(  Data.Sequence );

NewData.In = [Data.rnaEnergyMinus4to37 NewData.CodonBias NewData.Codon NewData.CodonCount];

load('Kudla_FinalModels.mat')

end

%%

for iOverEnsembleModels = 1:100
    
    [ResInd(:,iOverEnsembleModels)] = SVMPredictArm ( SVMModel.OuterLoopRep( iOverEnsembleModels ).Model, NewData.In  );

end

ResEns = mean( ResInd,2);

end

function [ out] = SVMPredictArm( SVMModel, RowArray  )

% calculates results for SVM with LIBSVM format

gamma = SVMModel.Parameters(4);

for iOverRowArray =1: size( RowArray , 1)
    
    Sub = repmat( RowArray( iOverRowArray,: ) ,size(SVMModel.SVs,1),1) - SVMModel.SVs;
    
    SqrModulous = sum( Sub.^2 ,2);
    
    Kernel = exp( - gamma .* SqrModulous);  % kernel is RBF
    
    out(iOverRowArray,1) = SVMModel.sv_coef' * Kernel - SVMModel.rho;
    
end

end
