function [ CodonSeq, CodonMat ] = nt2codons( Input )

% This function converts the nucleotide sequence into codons and attributes a number to each codon

% Input is a row vector with a sequence of ATCG bases

% CodonMat contains a row for each protein encoding and the columns are the features
% CodonSeq has the same information as CodonMat but in a different data structure 

% By changing the the values 1234 atributted to ACGT it is possible to change the value given to each codon

maxSize = 0;
for iOverGenes = 1:size(Input,1)
    
    Temp = Input{iOverGenes};
    
    Temp( Temp =='A' | Temp =='a' ) = '0';
    Temp( Temp =='C' | Temp =='c') = '1';
    Temp( Temp =='G' | Temp =='g') = '2';
    Temp( Temp =='T' | Temp =='t') = '3';
    Temp = str2num(Temp');
    Temp = reshape( Temp , 3, size( Temp,1 )/3 )';
    Temp = ( sum( Temp.* repmat([16 4 1], size( Temp,1 ), 1) ,2) )' ;
    
    CodonSeq{iOverGenes} = Temp+1;
    
    if size(Temp,2) > maxSize
        maxSize = size(Temp,2);
    end    
    
end

CodonMat = zeros( size(Input,1) , maxSize );

for iOverGenes = 1:size(Input,1)
    CodonMat( iOverGenes, 1:size( CodonSeq{iOverGenes},2 ) ) = CodonSeq{iOverGenes};
end    


end

