function PLSWithNFoldCrossVal(InputIn,DesiredIn,TrainValPerc,kFold,Dir,Obs)

% This function is called by ScriptToRunPLSWithNFoldCrossVal2.m

% Input is Obs x Var
% TrainValPerc is a percentage between 0 and 1
% kFold is number of folds
% Dir is a string with the directory of the calculations it terminates with "_"
% Obs is the observations


% Always check the assignment of patterns in functions nFoldCrossValidation and CalculateLastNet

clc


CalculateAllFolds=1;   %might be useful in the future

Input=InputIn;
Desired=DesiredIn;

% [Input, InputMean, InputStd] = zscore(Input);
% [Desired, DesMean, DesStd] = zscore(Desired);

mkdir(Dir)

save([Dir '\Data.mat'],'Input','Desired')

DirEntry=pwd;


%% calculate all kFold

CalcLastNet = 0;
for iOverFold=1:kFold
    PLSData(iOverFold).Param = nFoldCrossValidation( size(Input,1) , TrainValPerc, kFold, iOverFold);
    cd(DirEntry);
    PLSData(iOverFold).Res = CalculatePLSRegress( Input, Desired, PLSData, iOverFold, CalcLastNet);
end

%% read all results in directories

for iOverFold=1:kFold
    PLSData(1).AllMSEVal(iOverFold,:) = PLSData(iOverFold).Res.MSEVal;
end

PLSData(1).AllMSEVal = sum(PLSData(1).AllMSEVal) / kFold;

IndexBestComp = find( PLSData(1).AllMSEVal == min(PLSData(1).AllMSEVal) );  
PLSData(1).BestComp = PLSData(1).Res.Comp( IndexBestComp );                 

PLSData(1).AllValY = zeros( size(PLSData(iOverFold).Param.RowForValSet) );
PLSData(1).AllValY( PLSData(iOverFold).Param.RowForTestSet,: ) = NaN;  % the test set entries are nans

for iOverFold=1:kFold
    PLSData(1).AllValY( PLSData(iOverFold).Param.RowForValSet,: ) = PLSData(iOverFold).Res.ValY(:,IndexBestComp); 
    PLSData(iOverFold).Res.TrainY = PLSData(iOverFold).Res.TrainY(:,IndexBestComp);                
    PLSData(iOverFold).Res.ValY   = PLSData(iOverFold).Res.ValY  (:,IndexBestComp);                
    PLSData(iOverFold).Res.TestY  = PLSData(iOverFold).Res.TestY (:,IndexBestComp);                
    PLSData(iOverFold).Res.Obs    = 'TrainY, ValY, TestY are for the Best Component';
end



%% calculate final model

cd(DirEntry);
CalcLastNet = 1;
PLSData(1).LastNet = CalculatePLSRegress( Input, Desired, PLSData, 0, CalcLastNet);

PLSData(1).Obs = Obs;
 
save([Dir '\PLSData.mat'],'PLSData')

cd(DirEntry)

end



function Param = nFoldCrossValidation( SizeInput1, TrainValPerc, kFold, iOverFold)

% calculates one of the kfolds

%%

if kFold>1
    nPatternsTrain=fix(SizeInput1*TrainValPerc)*(kFold-1)/kFold;
elseif kFold==1
    error('kfold cannot be equal to one') 
end

nPatternsVal=fix(SizeInput1*TrainValPerc)-nPatternsTrain;
nPatternsTest=SizeInput1-(nPatternsTrain+nPatternsVal);


%%   Code for choosing non consecutive patterns for a fold

nSubFolds=nPatternsVal;

%    nPatternsPerSubFold=SizeInput1/nSubFolds; % this includes train, val and test
nPatternsPerSubFold = (SizeInput1-nPatternsTest)/nSubFolds; % this includes train and val, test is left out

if rem(nPatternsTrain,nSubFolds)~=0 | fix(nPatternsTrain)~=nPatternsTrain
    error('Problem with number of patterns 1')
end

if rem(nPatternsVal,nSubFolds)~=0 | fix(nPatternsVal)~=nPatternsVal
    error('Problem with number of patterns 2')
end

%     if rem(nPatternsTest,nSubFolds)~=0 | fix(nPatternsTest)~=nPatternsTest
%         error('Problem with number of patterns 3')
%     end

if fix(nPatternsTest)~=nPatternsTest
    error('Problem with number of patterns 4')
end

Param.RowForTrainSet = logical(zeros(SizeInput1,1));
Param.RowForValSet = logical (zeros(SizeInput1,1));
for iOverSubFold=1:nSubFolds
    IndexSubFold=(iOverSubFold-1)*nPatternsPerSubFold;
    Param.RowForTrainSet(IndexSubFold+1:IndexSubFold+(nPatternsTrain+nPatternsVal)/nSubFolds,1)=1;
    Param.RowForTrainSet(IndexSubFold+iOverFold)=0;
    Param.RowForValSet(IndexSubFold+iOverFold)=1;
end

Param.RowForTestSet= logical(1-(Param.RowForTrainSet + Param.RowForValSet));
%
% %%%%  end Code for choosing non consecutive patterns for a fold
%%


if sum(abs(Param.RowForTrainSet)+abs(Param.RowForValSet)+abs(Param.RowForTestSet))~=SizeInput1
    error('There are repeated patterns in the sets')
end


end

function Res = CalculatePLSRegress( Input, Desired, PLSData, iOverFold, CalcLastNet)
%%
if CalcLastNet
    RowForTrainSet = logical ( PLSData(1).Param.RowForTrainSet + PLSData(1).Param.RowForValSet );
    RowForValSet = [];
    RowForTestSet = PLSData(1).Param.RowForTestSet;

    if sum(abs(RowForTrainSet)+abs(RowForTestSet)) ~= size(Input,1)
        error('There are repeated patterns in the sets')
    end
    
    CompArray = PLSData(1).BestComp(1);
else    
    RowForTrainSet = PLSData(iOverFold).Param.RowForTrainSet;
    RowForValSet = PLSData(iOverFold).Param.RowForValSet;
    RowForTestSet = PLSData(iOverFold).Param.RowForTestSet;
    
    if sum(abs(RowForTrainSet)+abs(RowForValSet)+abs(RowForTestSet)) ~= size(Input,1)
        error('There are repeated patterns in the sets')
    end
    
    CompArray = 1:min(size(Input(RowForTrainSet,:),1)-1,size(Input(RowForTrainSet,:),2));
end  

Res.TrainY = [];
Res.ValY = [];
Res.TestY = [];
Res.MSETrain = [];
Res.MSEVal = [];
Res.MSETest = [];

for iOverComp = CompArray
    
    [~,~,~,~,beta] = plsregress( Input(RowForTrainSet,:), Desired(RowForTrainSet,:), iOverComp );
    
    Res.TrainY = [Res.TrainY [ones(size( Input( RowForTrainSet,:),1),1) Input( RowForTrainSet,:)]*beta ];
    Res.ValY   = [Res.ValY   [ones(size( Input( RowForValSet,:),1),1) Input( RowForValSet,:)]*beta ];     
    Res.TestY  = [Res.TestY  [ones(size( Input( RowForTestSet,:),1),1) Input( RowForTestSet,:)]*beta ];  
    
    Res.MSETrain = [Res.MSETrain ( mean((Res.TrainY(:,end) - Desired( RowForTrainSet,:)).^2) )] ;       
    Res.MSEVal   = [Res.MSEVal ( mean((Res.ValY(:,end)   - Desired( RowForValSet,:)).^2) )] ;         
    Res.MSETest  = [Res.MSETest  ( mean((Res.TestY(:,end)  - Desired( RowForTestSet,:)).^2) )] ;    
        
end

Res.Comp = CompArray;

if CalcLastNet
    Res.BetaOut = beta;
end    

end




