function SVMWithNFoldCrossVal3(InputIn,DesiredIn,TrainValPerc,kFold,Dir,Par,Obs)

% This function is called from "ScriptToRunSVMWithNFoldCrossVal2.m"

% Please set SVM parameter values in function "GetPar" below



% Input is Obs x Var
% TrainValPerc is a percentage between 0 and 1
% kFold is number of folds it must be one if TrainValPerc < 1
% Dir is a string with the directory of the calculations it terminates with "_"
% Obs is the observations



clc


CalculateAllFolds=1;   % might be useful in the future

Input=InputIn;
Desired=DesiredIn;

% [Input, InputMean, InputStd] = zscore(Input);
% [Desired, DesMean, DesStd] = zscore(Desired);

  
mkdir(Dir)

%save([SaveDir '\Data.mat'],'Input','Desired') % Data is saved in ScriptToRunSVMWithNFoldCrossVal2

ParamString = GetPar( Par );

DirEntry=pwd;

SVMData(1).ParamString = ParamString;

%%
   
% Calculate all kFold
CalcLastNet = 0;
for iOverFold=1:kFold
    SVMData(iOverFold).Param = nFoldCrossValidation( size(Input,1) , TrainValPerc, kFold, iOverFold);
    cd(DirEntry);
    SVMData(iOverFold).Res = CalculateSVMRegress( Input, Desired, SVMData, iOverFold, CalcLastNet);
end

% Read all results fom kFolds

for iOverFold=1:kFold
    SVMData(1).AllMSEVal(iOverFold,:) = SVMData(iOverFold).Res.MSEVal;
end
    
    
%% Calculate final model

cd(DirEntry);

CalcLastNet = 1;
iOverFold = 1;
SVMData(1).LastNet = CalculateSVMRegress( Input, Desired, SVMData, iOverFold, CalcLastNet);

SVMData(1).Obs = Obs;
 
save([Dir '\SVMData.mat'],'SVMData')

cd(DirEntry)

end



function Param = nFoldCrossValidation( SizeInput1, TrainValPerc, kFold, iOverFold)

% calculates one of the kfolds

%%

if kFold>1
    %%
    nPatternsTrain=fix(SizeInput1*TrainValPerc)*(kFold-1)/kFold;
    
    nPatternsVal=fix(SizeInput1*TrainValPerc)-nPatternsTrain;
    nPatternsTest=SizeInput1-(nPatternsTrain+nPatternsVal);
    
    
    %   Code for choosing non consecutive patterns for a fold
    
    nSubFolds=nPatternsVal;
    
    %    nPatternsPerSubFold=SizeInput1/nSubFolds; % this includes train, val and test
    nPatternsPerSubFold = (SizeInput1-nPatternsTest)/nSubFolds; % this includes train and val, test is left out
    
    if rem(nPatternsTrain,nSubFolds)~=0 | fix(nPatternsTrain)~=nPatternsTrain
        error('Problem with number of patterns 1')
    end
    
    if rem(nPatternsVal,nSubFolds)~=0 | fix(nPatternsVal)~=nPatternsVal
        error('Problem with number of patterns 2')
    end
    
    %     if rem(nPatternsTest,nSubFolds)~=0 | fix(nPatternsTest)~=nPatternsTest
    %         error('Problem with number of patterns 3')
    %     end
    
    if fix(nPatternsTest)~=nPatternsTest
        error('Problem with number of patterns 4')
    end
    
    Param.RowForTrainSet = logical(zeros(SizeInput1,1));
    Param.RowForValSet = logical (zeros(SizeInput1,1));
    for iOverSubFold=1:nSubFolds
        IndexSubFold=(iOverSubFold-1)*nPatternsPerSubFold;
        Param.RowForTrainSet( IndexSubFold + 1:IndexSubFold +( nPatternsTrain + nPatternsVal )/ nSubFolds,1)=1;
        Param.RowForTrainSet( IndexSubFold + iOverFold )=0;
        Param.RowForValSet( IndexSubFold + iOverFold )=1;
    end
    
    Param.RowForTestSet= logical(1-(Param.RowForTrainSet + Param.RowForValSet));
    %
    %  end Code for choosing non consecutive patterns for a fold

end    

if sum(abs(Param.RowForTrainSet)+abs(Param.RowForValSet)+abs(Param.RowForTestSet))~=SizeInput1
    error('There are repeated patterns in the sets')
end


end

function Res = CalculateSVMRegress( Input, Desired, SVMData, iOverFold, CalcLastNet)
%%

LIBSVM_Dir = '..\libSVM\libsvm-3.19\matlab';

if CalcLastNet   
    RowForTrainSet = logical(SVMData(1).Param.RowForTrainSet + SVMData(1).Param.RowForValSet);
    RowForValSet = logical (zeros(size(SVMData(1).Param.RowForValSet)));
    RowForTestSet = SVMData(1).Param.RowForTestSet;
else 
    RowForTrainSet = SVMData(iOverFold).Param.RowForTrainSet;
    RowForValSet = SVMData(iOverFold).Param.RowForValSet;
    RowForTestSet = SVMData(iOverFold).Param.RowForTestSet;
end    
    
if sum(abs(RowForTrainSet)+abs(RowForValSet)+abs(RowForTestSet)) ~= size(Input,1)
    error('There are repeated patterns in the sets')
end
  

cd( LIBSVM_Dir  )
    
SVMModel =   svmtrain(   Desired(RowForTrainSet,:), Input(RowForTrainSet,:), SVMData(1).ParamString );

Res.TrainY = svmpredict( zeros(sum(RowForTrainSet),1) , Input(RowForTrainSet,:), SVMModel);
Res.ValY =   svmpredict( zeros(sum(RowForValSet),1)   , Input(RowForValSet,:)  , SVMModel);
Res.TestY =  svmpredict( zeros(sum(RowForTestSet),1)  , Input(RowForTestSet,:) , SVMModel);

Res.MSETrain = mean((Res.TrainY - Desired( RowForTrainSet,:)).^2);
Res.MSEVal =   mean((Res.ValY   - Desired( RowForValSet,:)).^2);
Res.MSETest =  mean((Res.TestY  - Desired( RowForTestSet,:)).^2);
 
if CalcLastNet
    Res.SVMModel = SVMModel;
end    


end


function String = GetPar(index);

CVal = [1e4, 1e5, 1e6, 1e7, 1e8]; 
gVal = [1e-7, 1e-6, 1e-5, 1e-4, 1e-3]; 
pVal = [ 1e-3, 1e-2, 1e-1, 1e0, 1e1  ];     

ParamComb = npermutek([1:5],3);

String = ['-s 3 -t 2 -c ' num2str(CVal(ParamComb(index,1))) ' -g ' num2str(gVal(ParamComb(index,2))) ' -p ' num2str(pVal(ParamComb(index,3)))];  
% s-SVM (3 e-SVR) t-Kernel(2 RBF) c (cost value) g(gamma value) p(epsilon value)

end



